SystemOrganization addCategory: #'CodeMine-Grammar'!
SystemOrganization addCategory: #'CodeMine-Text'!
SystemOrganization addCategory: #CodeMine!

Object subclass: #CMCallGraph
	instanceVariableNames: 'selectorsMap methods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMCallGraph class methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 20:32'!
fromCategory: aString
	"Compute call graph from categories beginning with aString"

	| callGraph |
	
	callGraph := CMCallGraph new.
	(CMCallGraphBuilder forGraph: callGraph) scanCategoryPrefix: aString.
	^callGraph! !

!CMCallGraph class methodsFor: 'as yet unclassified' stamp: 'toma 8/13/2015 15:09'!
fromCategory: aString excluding: aBlock
	"Compute call graph from categories beginning with aString"

	| callGraph |
	
	callGraph := CMCallGraph new.
	(CMCallGraphBuilder forGraph: callGraph) scanCategoryPrefix: aString excluding: aBlock.
	^callGraph! !

!CMCallGraph methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:54'!
addMethod: aCMMethod
	methods add: aCMMethod
	
	! !

!CMCallGraph methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:07'!
initialize 
	super initialize.
	selectorsMap := Dictionary new.
	methods := OrderedCollection new.! !

!CMCallGraph methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
methods
	^ methods! !

!CMCallGraph methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
methods: anObject
	methods := anObject! !

!CMCallGraph methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:14'!
selectorFor: aSymbol
	
	^selectorsMap
		at: aSymbol 
		ifAbsentPut: [ CMSelector named: aSymbol ]! !

!CMCallGraph methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
selectorsMap
	^ selectorsMap! !

!CMCallGraph methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
selectorsMap: anObject
	selectorsMap := anObject! !

Object subclass: #CMCallingRelation
	instanceVariableNames: 'previous next container selector backwardDeps forwardDeps'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:31'!
backwardDeps
	^ backwardDeps! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:31'!
backwardDeps: anObject
	backwardDeps := anObject! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
container
	^ container! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
container: anObject
	container := anObject! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:31'!
forwardDeps
	^ forwardDeps! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:31'!
forwardDeps: anObject
	forwardDeps := anObject! !

!CMCallingRelation methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 14:32'!
initialize

	backwardDeps := OrderedCollection new.
	forwardDeps := OrderedCollection new.! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
selector
	^ selector! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
selector: anObject
	selector := anObject! !

Object subclass: #CMCodeTopicModel
	instanceVariableNames: 'methodTexts featureSpace topicModel linkField'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Text'!

!CMCodeTopicModel methodsFor: 'as yet unclassified' stamp: 'toma 8/14/2015 18:42'!
computeLinkedTopics: numTopics forIterations: iterations
	
	| lda |

	lda := LDLinkedAllocator new.
	lda linkField: linkField;
		contextWeight: 0.3;
		featurePriors: (self featureSpace estimateFeaturePriors).
	
	self topicModel: (lda
		allocate: (self methodTexts values) 
		iterating: iterations 
		features: (self featureSpace featureCount) 
		topics: numTopics).
	^self topicModel
	
	! !

!CMCodeTopicModel methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:41'!
computeTopics: numTopics forIterations: iterations
	
	| lda |

	lda := LDAllocator new.
	self topicModel: (lda
		allocate: (self methodTexts values) 
		iterating: iterations 
		features: (self featureSpace featureCount) 
		topics: numTopics).
	^self topicModel
	
	! !

!CMCodeTopicModel methodsFor: 'as yet unclassified' stamp: 'toma 8/14/2015 17:38'!
extractTextFromCallGraph: aCallGraph
	"Collect all textual tokens from a call graph, eliminate non-frequent tokens
	and construct the feature space from the rest"

	| frequencies methodFeatures dependencies selectorMapping docId |
	frequencies := Dictionary new.
	methodFeatures := OrderedDictionary new.
	dependencies := Dictionary new.
	selectorMapping := Dictionary new.

	docId := 1.
	
	aCallGraph methods do: [ :method | | textVisitor currentSelector |
		currentSelector := method implementedSelector symbol.
		selectorMapping at: currentSelector ifAbsentPut: [OrderedCollection new].
		(selectorMapping at: currentSelector) add: docId.
			
		textVisitor := CMTextVisitor new.
		textVisitor onSelector: [ :position :targetSelector | 
			dependencies at: (docId @ position) put: targetSelector.
		].

		textVisitor visitNode: (method sourceTree).
		textVisitor literals do: [ :symbol |
			frequencies at: symbol put: (1 + (frequencies at: symbol ifAbsentPut: 0)).
		].
		methodFeatures at: method put: (textVisitor literals).
		docId := docId + 1.
	].

	methodFeatures keysAndValuesDo: [ :method :features | | reducedFeatures |
		reducedFeatures := features select: [:feature | (frequencies at: feature) > 2 ].
		self methodTexts
			at: method 
			put: (self featureSpace representAll: reducedFeatures).
		].

	"Unwrap 
		(d @ w) -> #selector
	and
		#selector -> d1, dw, ...
	to
		(d @ w) -> d1, d2, ..."

	linkField := LDLinkField new.
	dependencies keysAndValuesDo: [ :position :targetSelector |
		selectorMapping at: targetSelector ifPresent: [ :targetIds |
			targetIds do: [ :id |  linkField linkDocument: (position x) at: (position y) to: id ]]].! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:34'!
featureSpace
	^ featureSpace! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:34'!
featureSpace: anObject
	featureSpace := anObject! !

!CMCodeTopicModel methodsFor: 'as yet unclassified' stamp: 'toma 8/14/2015 18:42'!
initialize 
	
	methodTexts := OrderedDictionary new.
	featureSpace := LDDocumentFeatureSpace new.! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:30'!
methodTexts
	^ methodTexts! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:30'!
methodTexts: anObject
	methodTexts := anObject! !

!CMCodeTopicModel methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:53'!
sortedTopic: topic
	
	^(((self topicModel featuresForTopicAt: topic) 
		withIndexCollect: [ :value :index | 
			Array with: value with: index]) 
		sorted: [ :a :b | (a first) > (b first) ])
		collect: [ :pair | self featureSpace interpret: (pair second) ]
	! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:34'!
topicModel
	^ topicModel! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:34'!
topicModel: anObject
	topicModel := anObject! !

Object subclass: #CMDataDependency
	instanceVariableNames: 'name definition uses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

CMDataDependency subclass: #CMArgumentDependency
	instanceVariableNames: 'sources'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMArgumentDependency class methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:48'!
named: aString
	
	^(CMArgumentDependency new)
		name: aString;
		yourself.! !

!CMArgumentDependency methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:49'!
isArgument

	^ true! !

!CMArgumentDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:41'!
sources
	^ sources! !

!CMArgumentDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:41'!
sources: anObject
	sources := anObject! !

!CMDataDependency class methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 14:42'!
definedAt: aCallingRelation
	
	^(CMDataDependency new)
		definition: aCallingRelation;
		uses: OrderedCollection new;
		yourself.! !

!CMDataDependency class methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 14:43'!
named: aString definedAt: aCallingRelation
	
	^(CMDataDependency new)
		definition: aCallingRelation;
		uses: OrderedCollection new;
		name: aString;
		yourself.! !

!CMDataDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:41'!
definition
	^ definition! !

!CMDataDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:41'!
definition: anObject
	definition := anObject! !

!CMDataDependency methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:49'!
isArgument

	^ false! !

!CMDataDependency methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:49'!
isSelf

	^ false! !

!CMDataDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:43'!
name
	^ name! !

!CMDataDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:43'!
name: anObject
	name := anObject! !

!CMDataDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:41'!
uses
	^ uses! !

!CMDataDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:41'!
uses: anObject
	uses := anObject! !

CMDataDependency subclass: #CMSelfDependency
	instanceVariableNames: 'cls'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMSelfDependency class methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 17:22'!
newForClass: aClass

	^(CMSelfDependency new)
		cls: aClass;
		name: 'self';
		yourself! !

!CMSelfDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:34'!
cls
	^ cls! !

!CMSelfDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:35'!
cls: anObject
	cls := anObject! !

!CMSelfDependency methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:49'!
isSelf

	^ true! !

Object subclass: #CMMethod
	instanceVariableNames: 'implementedSelector callingRelations sourceTree'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
callingRelations
	^ callingRelations! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
callingRelations: anObject
	callingRelations := anObject! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
implementedSelector
	^ implementedSelector! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
implementedSelector: anObject
	implementedSelector := anObject! !

!CMMethod methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:35'!
initialize 
	super initialize.
	callingRelations := OrderedCollection new.! !

!CMMethod methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 17:26'!
newCallTo: aSelector 

	| call |
	call := CMCallingRelation new.
	call	selector: aSelector;
		container: self.
	aSelector senders add: call.
	self callingRelations add: call.
	^call
	! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 20:49'!
sourceTree
	^sourceTree! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 21:08'!
sourceTree: aSourceAst
	sourceTree := aSourceAst! !

Object subclass: #CMSelector
	instanceVariableNames: 'symbol implementors senders'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMSelector class methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:14'!
named: aSymbol

	^(CMSelector new)
		symbol: aSymbol;
		yourself! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
implementors
	^ implementors! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
implementors: anObject
	implementors := anObject! !

!CMSelector methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:13'!
initialize
	implementors := OrderedCollection new.
	senders := OrderedCollection new.! !

!CMSelector methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 21:08'!
newImplementor: aSourceAst
	
	| method |

	method := CMMethod new.
	method 
		implementedSelector: self;
		sourceTree: aSourceAst.
	self implementors add: method.
	^method
! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
senders
	^ senders! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
senders: anObject
	senders := anObject! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
symbol
	^ symbol! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
symbol: anObject
	symbol := anObject! !

Object subclass: #CXExpression
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

CXExpression subclass: #CXChoice
	instanceVariableNames: 'left right probability'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

!CXChoice methodsFor: 'accessing' stamp: 'toma 9/1/2015 18:28'!
left

	^ left! !

!CXChoice methodsFor: 'accessing' stamp: 'toma 9/1/2015 18:28'!
left: anObject

	left := anObject! !

!CXChoice methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 18:36'!
parseOn: aParseState

	^(self left parseOn: aParseState) 
		onSuccess: [ :result | 
			(aParseState statsFor: self) goLeft. 
			result ]
		onFailure: [
			(self right parseOn: aParseState) 
			].! !

!CXChoice methodsFor: 'accessing' stamp: 'toma 9/1/2015 18:28'!
probability

	^ probability! !

!CXChoice methodsFor: 'accessing' stamp: 'toma 9/1/2015 18:28'!
probability: anObject

	probability := anObject! !

!CXChoice methodsFor: 'accessing' stamp: 'toma 9/1/2015 18:28'!
right

	^ right! !

!CXChoice methodsFor: 'accessing' stamp: 'toma 9/1/2015 18:28'!
right: anObject

	right := anObject! !

!CXChoice methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 17:27'!
statClass
	^CXChoiceStat! !

CXExpression subclass: #CXString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

Object subclass: #CXParseResult
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

CXParseResult subclass: #CXFailure
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

!CXFailure methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 18:00'!
onSuccess: successBlock onFailure: failureBlock

	^ failureBlock value! !

!CXParseResult methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 18:29'!
onSuccess: successBlock onFailure: failureBlock
	^ successBlock value: self

	! !

CXParseResult subclass: #CXToken
	instanceVariableNames: 'data from to'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

Object subclass: #CXParseState
	instanceVariableNames: 'data position stats'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

!CXParseState methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:27'!
data

	^ data! !

!CXParseState methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:27'!
data: anObject

	data := anObject! !

!CXParseState methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:32'!
initialize

	super initialize.
	self position: 1.
	self stats: (Dictionary new).! !

!CXParseState methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:27'!
position

	^ position! !

!CXParseState methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:27'!
position: anObject

	position := anObject! !

!CXParseState methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:27'!
stats

	^ stats! !

!CXParseState methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:27'!
stats: anObject

	stats := anObject! !

!CXParseState methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 17:26'!
statsFor: anExpression

	^self stats at: anExpression ifAbsentPut: [anExpression statClass new].! !

Object subclass: #CXStat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

CXStat subclass: #CXChoiceStat
	instanceVariableNames: 'leftCount rightCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

!CXChoiceStat methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 17:22'!
goLeft
	self leftCount: (self leftCount + 1)! !

!CXChoiceStat methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 17:22'!
goRight
	self rightCount: (self rightCount + 1)! !

!CXChoiceStat methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 17:17'!
initialize
	super initialize.
	self
		leftCount: 0;
		rightCount: 0.	! !

!CXChoiceStat methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:14'!
leftCount

	^ leftCount! !

!CXChoiceStat methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:14'!
leftCount: anObject

	leftCount := anObject! !

!CXChoiceStat methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 17:20'!
probability
	| sum |
	"maximum likelihood estimate for P given collected statistics"
	sum := self leftCount + (self rightCount).
	(sum = 0)
		ifTrue: [	 ^ self  leftCount asFloat / sum ]
		ifFalse: [ ^ 0.5 ]
	! !

!CXChoiceStat methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:14'!
rightCount

	^ rightCount! !

!CXChoiceStat methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:14'!
rightCount: anObject

	rightCount := anObject! !

RBProgramNodeVisitor subclass: #CMCallGraphBuilder
	instanceVariableNames: 'currentMethod callGraph definitions currentSelf controlFlow currentStatement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMCallGraphBuilder class methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 18:02'!
forGraph: aCallGraph
	
	^(CMCallGraphBuilder new)
		callGraph: aCallGraph;
		yourself.
		! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:20'!
acceptMessageNode: aMessageNode
	| selector |


	selector := self callGraph selectorFor: aMessageNode selector.
	self currentStatement: (self currentMethod newCallTo: selector).
	super acceptMessageNode: aMessageNode.

	! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:50'!
acceptMethodNode: aMethodNode
	
	| selector |
	selector := self callGraph selectorFor: (
		aMethodNode selector asSymbol).
	self currentMethod: (selector newImplementor: aMethodNode).
	self definitions: Dictionary new.
	self definitions at: 'self' put: (self currentSelf).
.
	aMethodNode argumentNames do: [ :arg |
		self definitions at: arg put: (CMArgumentDependency named: arg)].

	self callGraph methods add: (self currentMethod).
	super acceptMethodNode: aMethodNode.! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/12/2015 18:02'!
callGraph
	^ callGraph! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/12/2015 18:02'!
callGraph: anObject
	callGraph := anObject! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:02'!
controlFlow
	^ controlFlow! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:02'!
controlFlow: anObject
	controlFlow := anObject! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/12/2015 18:02'!
currentMethod
	^ currentMethod! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/12/2015 18:02'!
currentMethod: anObject
	currentMethod := anObject! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:28'!
currentSelf
	^ currentSelf! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:28'!
currentSelf: anObject
	currentSelf := anObject! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:18'!
currentStatement
	^ currentStatement! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:18'!
currentStatement: anObject
	currentStatement := anObject! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:02'!
definitions
	^ definitions! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:02'!
definitions: anObject
	definitions := anObject! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:02'!
initialize 

	super initialize.
	definitions := Dictionary new.! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/14/2015 11:57'!
scanCategoryPrefix: aString

	^self scanCategoryPrefix: aString excluding: [ :class | false ]
! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/13/2015 15:33'!
scanCategoryPrefix: aString excluding: aBlock

	((Object allSubclasses
		select: [ :class |
			class category ifNil: [false] ifNotNilDo: [:cat | cat beginsWith: aString]])
		select: [ :class |
			(aBlock value: class) not]) 
		do: [: class |
			self scanClass: class].
! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:37'!
scanClass: aClass

	self currentSelf: (CMSelfDependency newForClass: aClass).

	aClass methodDict valuesDo: [ :method | 	| node |
		node := RBParser parseMethod: (method getSource asString).
		self visitNode: node.
	]
! !

RBProgramNodeVisitor subclass: #CMTextVisitor
	instanceVariableNames: 'literals onSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Text'!

!CMTextVisitor class methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 23:18'!
normalize: aString
	"Splits a camel case selector into lowercase atoms without colons"
	
	| tokens lastIndex string |

	string := aString.
	tokens := OrderedCollection new.
	lastIndex := 1.
	string withIndexDo: [ :char :index |
		(char isUppercase or: [char isLetter not]) ifTrue: [
			(index > 1) ifTrue: [
				tokens add: (string copyFrom: lastIndex to: (index - 1)) asLowercase.
			].
			lastIndex := index.
		]
	].
	(lastIndex <= (string size)) ifTrue: [
		tokens add: (string copyFrom: lastIndex to: (string size)) asLowercase.
	].
	^(tokens collect: [:token | token onlyLetters]) select: [:token | token size >= 3]! !

!CMTextVisitor methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:26'!
acceptLiteralNode: aLiteralNode
	| value |
	super acceptLiteralNode: aLiteralNode.
	value := aLiteralNode token value.
	(value isString)
		ifTrue: [(CMTextVisitor normalize: value) do: [:atom | self literals add: atom]]
	! !

!CMTextVisitor methodsFor: 'as yet unclassified' stamp: 'toma 8/18/2015 18:25'!
acceptMessageNode: aMessageNode 
	
	super acceptMessageNode: aMessageNode.
	(CMTextVisitor normalize: (aMessageNode selector)) do: [ :atom |
		self literals add: atom.
		self onSelector value: (self literals size) value: (aMessageNode selector)].
	self literals add: (aMessageNode selector)! !

!CMTextVisitor methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:26'!
acceptMethodNode: aMethodNode
	
	super acceptMethodNode: aMethodNode.
	(CMTextVisitor normalize: (aMethodNode selector)) do: [ :atom |
		self literals add: atom]! !

!CMTextVisitor methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:26'!
acceptVariableNode: aVariableNode

	super acceptVariableNode: aVariableNode.
	(CMTextVisitor normalize: (aVariableNode name)) do: [ :atom |
		self literals add: atom]
	! !

!CMTextVisitor methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 21:36'!
initialize 

	super initialize.
	literals := OrderedCollection new.! !

!CMTextVisitor methodsFor: 'accessing' stamp: 'toma 8/12/2015 21:38'!
literals
	^ literals! !

!CMTextVisitor methodsFor: 'accessing' stamp: 'toma 8/12/2015 21:38'!
literals: anObject
	literals := anObject! !

!CMTextVisitor methodsFor: 'accessing' stamp: 'toma 8/14/2015 16:39'!
onSelector
	^ onSelector! !

!CMTextVisitor methodsFor: 'accessing' stamp: 'toma 8/14/2015 16:39'!
onSelector: anObject
	onSelector := anObject! !
